/**
 * Copyright Copyright 2010 Simon Andrews
 *
 *    This file is part of Difference Tracker.
 *
 *    Difference Tracker is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Difference Tracker is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with Difference Tracker; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.DifferenceTracker.Results;

import uk.ac.bbsrc.babraham.DifferenceTracker.SimpleStats;
import ij.ImagePlus;

public class DifferenceResults {

	private Double [] staticIntensity;
	private Double [] movingIntensity;
	private Integer [] staticCount;
	private Integer [] movingCount;
	private double pixelCount;
	
	public DifferenceResults (ImagePlus imp) {
		pixelCount = (imp.getHeight()*imp.getWidth())/1000d;
		
		int frameCount = imp.getStackSize();
		
		staticIntensity = new Double [frameCount];
		movingIntensity = new Double [frameCount];
		staticCount = new Integer [frameCount];
		movingCount = new Integer [frameCount];
		
	}
	
	public void setStaticIntensity (double intensity, int slice) {
		if (staticIntensity[slice-1] != null) throw new IllegalArgumentException("Static intensity has already been set for slice "+slice);
		
		staticIntensity[slice-1] = intensity;
	}

	public void setMovingIntensity (double intensity, int slice) {
		if (movingIntensity[slice-1] != null) throw new IllegalArgumentException("Moving intensity has already been set for slice "+slice);
		
		movingIntensity[slice-1] = intensity;
	}

	public void setStaticCount (int count, int slice) {
		if (staticCount[slice-1] != null) throw new IllegalArgumentException("Static count has already been set for slice "+slice);
		
		staticCount[slice-1] = count;
	}

	public void setMovingCount (int count, int slice) {
		if (movingCount[slice-1] != null) throw new IllegalArgumentException("Moving count has already been set for slice "+slice);
		
		movingCount[slice-1] = count;
	}
	
	public double averageStaticIntensity () {
		return SimpleStats.mean(staticIntensity);
	}
	
	public double averageStaticIntensityPer1000Pixel () {
		return averageStaticIntensity()/pixelCount;
	}
	
	public double averageMovingIntensity () {
		return SimpleStats.mean(movingIntensity);
	}
	
	public double averageMovingIntensityPer1000Pixel () {
		return averageMovingIntensity()/pixelCount;
	}
	
	public double averageStaticCount () {
		return SimpleStats.mean(staticCount);
	}
	
	public double averageStaticCountPer1000Pixel () {
		return averageStaticCount()/pixelCount;
	}
	
	public double averageMovingCount () {
		return SimpleStats.mean(movingCount);
	}
	
	public double averageMovingCountPer1000Pixel (){
		return averageMovingCount()/pixelCount;
	}
	
	public double movingIntensityPercentage () {
		Double [] p = new Double[staticIntensity.length];
		
		for (int i=0;i<staticIntensity.length;i++) {
			if (staticIntensity[i] == null || movingIntensity[i] == null) continue;
			
			p[i] = 100 * (movingIntensity[i] / (staticIntensity[i]+movingIntensity[i]));
		}
		
		return SimpleStats.mean(p);
	}

	public double movingCountPercentage () {
		Double [] p = new Double[staticCount.length];
		
		for (int i=0;i<staticCount.length;i++) {
			if (staticCount[i] == null || movingCount[i] == null) continue;
			
			p[i] = 100 * ((double)movingCount[i] / (staticCount[i]+movingCount[i]));
		}
		
		return SimpleStats.mean(p);
	}

	
}
