/**
 * Copyright Copyright 2010 Simon Andrews
 *
 *    This file is part of Difference Tracker.
 *
 *    Difference Tracker is free software; you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation; either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    Difference Tracker is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with Difference Tracker; if not, write to the Free Software
 *    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
package uk.ac.bbsrc.babraham.DifferenceTracker;
import ij.ImagePlus;
import ij.process.ImageProcessor;

import java.awt.Color;
import java.util.Vector;

public class TrackedParticle implements Comparable<TrackedParticle> {
	
	/**
	 * This class represents a single particle as it is tracked
	 * through time.  Since these tracked particles will be 
	 * stored to calculate statistics we don't want to put too
	 * much information in here, so just a set of coordinates
	 * linked to frames should be enough
	 */

	private Vector<XYPosition> positions = new Vector<XYPosition>();
	private int slice;
	private XYPosition predicted = null;
	private int totalVolume = 0;
	
	public static final int UP = 1;
	public static final int DOWN = 2;
	public static final int LEFT = 3;
	public static final int RIGHT = 4;
	
	public TrackedParticle (Particle p, int slice) {
		positions.add(p.getCentrePosition());
		totalVolume += p.volume();
		predictNextPosition();
		this.slice = slice;
	}
	
	public double wantsParticle (Particle p) {
		
		/**
		 * Calculates the distance from the supplied particle to the
		 * next predicted position of this track.
		 */

		XYPosition xy = p.getCentrePosition();
		
		double distance = Math.sqrt(Math.pow(xy.x-predicted.x, 2)+Math.pow(xy.y-predicted.y,2));
		
//		System.out.println("Distance from "+xy.x+","+xy.y+" to "+predicted.x+","+predicted.y+" is "+distance);
		
		return distance;
	}
	
	public void takeParticle (Particle p) {
		
		/**
		 * Adds the supplied particle as the next position on this track.
		 * The decision of which tracked particle gets which new particle
		 * is handled in the TrackedParticleCollection as each individual
		 * particle doesn't have enough information to do this.
		 * 
		 */
		
		positions.add(p.getCentrePosition());
		totalVolume += p.volume();
		predictNextPosition();
	}
	
	public XYPosition getOverallDirection () {
		int dx = 0;
		int dy = 0;

		XYPosition [] pos = positions.toArray(new XYPosition[0]);

		dx = pos[pos.length-1].x - pos[0].x ;
		dy = pos[pos.length-1].y - pos[0].y;
		
		return new XYPosition(dx,dy);
		
	}
	
	public Color getTrackColor () {
		
		/*
		 * Track colours are:
		 * 
		 * Red:    Moving down
		 * Green:  Moving up
		 * Blue:   Moving left
		 * Yellow: Moving right
		 */
		
		switch (getDirectionOfMovement()) {
		case UP: return Color.GREEN;
		case DOWN: return Color.RED;
		case LEFT: return Color.BLUE;
		case RIGHT: return Color.YELLOW;
		default: return Color.white;
		}
		
	}

	public int getDirectionOfMovement () {
		XYPosition dxy = getOverallDirection();

		// We're reducing this to classifying only as left or right
		if (dxy.x > 0) return RIGHT;
		else return LEFT;

		
//		if (Math.abs(dxy.x) > Math.abs(dxy.y)) {
//			if (dxy.x > 0) return RIGHT;
//			else return LEFT;
//		}
//		else {
//			if (dxy.y > 0) return DOWN;
//			else return UP;
//		}
		
	}
	
	public void addTrackToImagePlus (ImagePlus imp) {
		
		/**
		 * Annotates an imageplus object with the track which is
		 * followed by this set of particles.
		 */
		
		XYPosition [] pos = positions.toArray(new XYPosition[0]);
		
//		System.out.println("Tracking particle from slice "+slice+" to "+(slice+(pos.length-1)));
		
		for (int p=0;p<pos.length-1;p++) {
//			System.out.println("Drawing position "+p);
			int thisSlice = slice+p+1;
			
			while (thisSlice < slice+pos.length) {
//				System.out.println("Drawing on slice "+thisSlice);
				ImageProcessor proc = imp.getImageStack().getProcessor(thisSlice);
								
				proc.setColor(getTrackColor());
				proc.drawLine(pos[p].x, pos[p].y, pos[p+1].x, pos[p+1].y);
//				System.out.println("Drawing line from "+pos[p].x+","+pos[p].y+" to "+ pos[p+1].x +","+ pos[p+1].y);
				thisSlice++;
			}
		}		
		
	}
	
	public double distanceMovedInSlice (int s) {
		if (! movedInSlice(s)) throw new IllegalArgumentException("This particle wasn't moving in slice "+s);
		
		XYPosition [] pos = positions.toArray(new XYPosition[0]);

//		System.out.println("Looking for distance moved in "+s+" length is "+length()+" slice is "+slice+" pos lenth is "+pos.length);
		
		return Math.sqrt(Math.pow(pos[s-slice].x-pos[s-(slice+1)].x, 2)+Math.pow(pos[s-slice].y-pos[s-(slice+1)].y,2));
		
	}
	
	public double distanceMoved () {
		/**
		 * Returns the total distance moved by this particle over all of the
		 * tracked positions
		 */
		
		double distance = 0;
		XYPosition [] pos = positions.toArray(new XYPosition[0]);
		
		for (int i=1;i<pos.length;i++) {
			
			distance += Math.sqrt(Math.pow(pos[i].x-pos[i-1].x, 2)+Math.pow(pos[i].y-pos[i-1].y,2));
			
		}
		
		return distance;
		
	}
	
	public double averageVolume () {
		return (double)totalVolume/length();
	}

	public boolean presentInSlice (int s) {
		if (s>=slice && s<slice+length()) {
			return true;
		}
		return false;
	}
	
	public boolean movedInSlice (int s) {
		if (s>=slice+1 && s<=slice+(length()-1)) {
			return true;
		}
		return false;
	}

	
	public boolean wasInSlice (int s) {
		if (s>=slice && s<slice+length()) {
			return true;
		}
		return false;
	}
	
	public double maxSpeed () {
		
		/**
		 * Returns the maximum movement observed in any slice
		 */
		
		double maxDistance = 0;
		XYPosition [] pos = positions.toArray(new XYPosition[0]);
		
		for (int i=1;i<pos.length;i++) {
			
			double thisDistance = Math.sqrt(Math.pow(pos[i].x-pos[i-1].x, 2)+Math.pow(pos[i].y-pos[i-1].y,2));
			if (thisDistance > maxDistance) maxDistance = thisDistance;
		}
		
		return maxDistance;
		
	}
	
	public int length () {
		
		/**
		 * Says how many tracked positions are currently held
		 */
		
		return positions.size();
	}
	
	public XYPosition prediction() {
		
		/**
		 * Provides the coordinates of the next predicted position for
		 * this track
		 */
		
		return predicted;
	}
	
	public int startSlice () {
		
		/**
		 * Says on which slice of the stack this track begins
		 */
		
		return slice;
	}
	
	public int endSlice () {
		return slice+(positions.size()-1);
	}
	
	public XYPosition positionInSlice (int s) {
		if (! wasInSlice(s)) {
			return null;
		}
		
		return positions.elementAt(s-slice);
	}
	
	public XYPosition startPosition () {
		return positions.elementAt(0);
	}
	
	public XYPosition endPosition () {
		return positions.elementAt(positions.size()-1);
	}

	private void predictNextPosition () {
			
		/**
		 * Works out a putative next position for this particle based on
		 * a simple extrapolation of the last difference observed.  This means
		 * that we don't handle sudden accelerations, decelerations or changes
		 * of direction, but that's proabably OK.
		 */
		
		//TODO: Could we try using a weighted average system to incorporate
		// information from all previous positions?  Would this help?
		
		XYPosition [] previousPositions = positions.toArray(new XYPosition[0]);

		if (previousPositions.length==1) {
			predicted = new XYPosition(previousPositions[0].x,previousPositions[0].y);
		}
		else if (previousPositions.length >=2) {
			XYPosition lastPosition = previousPositions[previousPositions.length-1];
			XYPosition twoAgo = previousPositions[previousPositions.length-2];
			
			XYPosition nextPosition = new XYPosition(lastPosition.x + (lastPosition.x-twoAgo.x),lastPosition.y + (lastPosition.y-twoAgo.y));
			
//			System.out.println("From "+twoAgo.x+","+twoAgo.y+" to "+lastPosition.x+","+lastPosition.y+" predicted "+nextPosition.x+","+nextPosition.y);
			
			predicted = nextPosition;
		}
		else {
			System.err.println("Supposedly impossible number of positions "+previousPositions.length+" found");
		}
		
		
	}

	public int compareTo(TrackedParticle o) {
		return o.length() - length();
	}

}
